"use strict";

import { createHighlighterCore } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import React from 'react';
import { ActivityIndicator, ScrollView, Text, View } from 'react-native';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
// Maintain a cache of loaded languages and themes
const languageCache = new Map();
const themeCache = new Map();

// Singleton highlighter instance
let highlighterInstance = null;
async function loadLanguage(lang) {
  if (languageCache.has(lang)) {
    return languageCache.get(lang);
  }
  try {
    const module = await import(`shiki/langs/${lang}.mjs`);
    languageCache.set(lang, module.default);
    return module.default;
  } catch {
    // Fallback to plaintext if language not found
    console.warn(`Language '${lang}' not found, falling back to plaintext`);
    return {
      name: 'plaintext',
      scopeName: 'source.plaintext',
      patterns: []
    };
  }
}
async function loadTheme(themeName) {
  if (themeCache.has(themeName)) {
    return themeCache.get(themeName);
  }
  try {
    const module = await import(`shiki/themes/${themeName}.mjs`);
    themeCache.set(themeName, module.default);
    return module.default;
  } catch {
    throw new Error(`Theme '${themeName}' not found`);
  }
}
async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine()
    });
  }
  return highlighterInstance;
}
export const SyntaxHighlighter = ({
  children,
  language,
  theme,
  textStyle,
  scrollViewProps,
  containerStyle
}) => {
  const [tokens, setTokens] = React.useState([]);
  const [backgroundColor, setBackgroundColor] = React.useState('#0d1117');
  const [loading, setLoading] = React.useState(true);
  const [error, setError] = React.useState(null);
  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true);

        // Load language and theme
        const [langData, themeData] = await Promise.all([loadLanguage(language), loadTheme(theme)]);

        // Get highlighter and load language/theme
        const highlighter = await getHighlighter();
        await highlighter.loadLanguage(langData);
        await highlighter.loadTheme(themeData);

        // Highlight the code
        const result = highlighter.codeToTokens(children, {
          lang: language,
          theme
        });
        setTokens(result.tokens);
        setBackgroundColor(result.bg || '#0d1117');
        setError(null);
      } catch (err) {
        setError(err instanceof Error ? err.message : 'Failed to highlight code');
        console.error('Highlighting error:', err);
      } finally {
        setLoading(false);
      }
    };
    highlight();
  }, [children, language, theme]);
  const renderToken = (token, index) => {
    const style = {
      color: token.color || '#c9d1d9',
      ...(token.fontStyle === 1 && {
        fontStyle: 'italic'
      }),
      ...(token.fontStyle === 2 && {
        fontWeight: 'bold'
      })
    };
    return /*#__PURE__*/_jsx(Text, {
      style: [textStyle, style],
      children: token.content
    }, `${token.offset}-${index}`);
  };
  const renderLine = (lineTokens, index) => /*#__PURE__*/_jsx(View, {
    style: {
      flexDirection: 'row',
      flexWrap: 'wrap'
    },
    children: lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
  }, index);
  if (loading) {
    return /*#__PURE__*/_jsx(View, {
      style: [{
        backgroundColor,
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsx(ActivityIndicator, {})
    });
  }
  if (error) {
    return /*#__PURE__*/_jsx(View, {
      style: [{
        backgroundColor,
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsxs(Text, {
        style: {
          color: '#ff6b6b'
        },
        children: ["Error:", error]
      })
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    style: [{
      backgroundColor
    }, scrollViewProps?.style],
    contentContainerStyle: [containerStyle],
    children: /*#__PURE__*/_jsx(View, {
      onStartShouldSetResponder: () => true,
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
};
export default SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map