import type { LanguageRegistration, ThemedToken, ThemeRegistration } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, Text, type TextStyle, View, type ViewStyle } from 'react-native'

export interface SyntaxHighlighterProps {
  children: string
  language: string
  theme: string
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
}

// Maintain a cache of loaded languages and themes
const languageCache = new Map<string, LanguageRegistration>()
const themeCache = new Map<string, ThemeRegistration>()

// Singleton highlighter instance
let highlighterInstance: Awaited<ReturnType<typeof createHighlighterCore>> | null = null

async function loadLanguage(lang: string) {
  if (languageCache.has(lang)) {
    return languageCache.get(lang)!
  }

  try {
    const module = await import(`shiki/langs/${lang}.mjs`)
    languageCache.set(lang, module.default)
    return module.default
  }
  catch {
    // Fallback to plaintext if language not found
    console.warn(`Language '${lang}' not found, falling back to plaintext`)
    return {
      name: 'plaintext',
      scopeName: 'source.plaintext',
      patterns: [],
    }
  }
}

async function loadTheme(themeName: string) {
  if (themeCache.has(themeName)) {
    return themeCache.get(themeName)!
  }

  try {
    const module = await import(`shiki/themes/${themeName}.mjs`)
    themeCache.set(themeName, module.default)
    return module.default
  }
  catch {
    throw new Error(`Theme '${themeName}' not found`)
  }
}

async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
    })
  }
  return highlighterInstance
}

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ children, language, theme, textStyle, scrollViewProps, containerStyle }) => {
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])
  const [backgroundColor, setBackgroundColor] = React.useState<string>('#0d1117')
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)

  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true)

        // Load language and theme
        const [langData, themeData] = await Promise.all([loadLanguage(language), loadTheme(theme)])

        // Get highlighter and load language/theme
        const highlighter = await getHighlighter()
        await highlighter.loadLanguage(langData)
        await highlighter.loadTheme(themeData)

        // Highlight the code
        const result = highlighter.codeToTokens(children, {
          lang: language,
          theme,
        })

        setTokens(result.tokens)
        setBackgroundColor(result.bg || '#0d1117')
        setError(null)
      }
      catch (err) {
        setError(err instanceof Error ? err.message : 'Failed to highlight code')
        console.error('Highlighting error:', err)
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [children, language, theme])

  const renderToken = (token: ThemedToken, index: number) => {
    const style = {
      color: token.color || '#c9d1d9',
      ...(token.fontStyle === 1 && { fontStyle: 'italic' as const }),
      ...(token.fontStyle === 2 && { fontWeight: 'bold' as const }),
    }

    return (
      <Text key={`${token.offset}-${index}`} style={[textStyle, style]}>
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => (
    <View key={index} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
      {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
    </View>
  )

  if (loading) {
    return (
      <View style={[{ backgroundColor, padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ backgroundColor, padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor }, scrollViewProps?.style]} contentContainerStyle={[containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
